package com.ruoyi.file.controller;

import com.ruoyi.file.domain.FileUpdateInfo;
import com.ruoyi.file.service.FileUpdateInfoService;
import com.usky.common.core.bean.ApiResult;
import com.usky.common.core.bean.CommonPage;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/api/update")
public class FileUpdateInfoController {

    // 从配置文件中读取脚本名称
    @Value("${file.linuxrestart}")
    private String linuxContralScript;

    @Value("${file.windowsstop}")
    private String windowsStopScript;

    @Value("${file.windowsstart}")
    private String windowsStartScript;

    @Autowired
    private FileUpdateInfoService fileUpdateInfoService;

    @PostMapping("/add")
    public ApiResult<String> addFileToDatabase(@RequestBody FileUpdateInfo fileUpdateInfo) {
        try {
            fileUpdateInfoService.addFileToDatabase(fileUpdateInfo);
            return ApiResult.success("文件信息录入成功");
        } catch (Exception e) {
            return ApiResult.error("文件信息录入失败: " + e.getMessage());
        }
    }

    @PostMapping("/checkById")
    public ApiResult<String> checkFileUpdate(@RequestParam Long id) {
        try {
            boolean isUpdateRequired = fileUpdateInfoService.checkFileUpdate(id);
            return ApiResult.success(isUpdateRequired ? "需要更新" : "不需要更新");
        } catch (Exception e) {
            return ApiResult.error("检查文件更新失败: " + e.getMessage());
        }
    }

    @PostMapping("/reloadById")
    public ApiResult<String> performFileUpdate(@RequestParam Long id) {
        try {
            fileUpdateInfoService.performFileUpdate(id);
            return ApiResult.success("文件更新成功");
        } catch (Exception e) {
            return ApiResult.error("文件更新失败: " + e.getMessage());
        }
    }

    @GetMapping("/query")
    public ApiResult<CommonPage<FileUpdateInfo>> queryFiles(
            @RequestParam(required = false) String fileName,
            @RequestParam(required = false, defaultValue = "1") int current,
            @RequestParam(required = false, defaultValue = "10") int size) {
        try {
            List<FileUpdateInfo> files;
            if (fileName == null || fileName.trim().isEmpty()) {
                files = fileUpdateInfoService.getAllFiles();
            } else {
                files = fileUpdateInfoService.getFilesByFileNameContaining(fileName);
            }

            CommonPage<FileUpdateInfo> commonPage = new CommonPage<>(
                    files.subList((current - 1) * size, Math.min(current * size, files.size())),
                    files.size(),
                    size,
                    current
            );
            return ApiResult.success(commonPage);
        } catch (Exception e) {
            return ApiResult.error("查询文件信息失败: " + e.getMessage());
        }
    }

    @DeleteMapping("/deleteById")
    @ApiOperation(value = "删除文件信息", notes = "根据文件 ID 删除指定文件信息")
    public ApiResult<String> deleteFileById(@RequestParam Long id) {
        try {
            fileUpdateInfoService.deleteFileById(id);
            return ApiResult.success("文件信息删除成功");
        } catch (Exception e) {
            return ApiResult.error("删除文件信息失败: " + e.getMessage());
        }
    }

    @PostMapping("/controlById")
    @ApiOperation(value = "控制服务", notes = "根据文件 ID 和操作类型控制服务（启动、停止、重启）")
    public ApiResult<String> controlServiceById(@RequestParam Long id, @RequestParam String action) {
        try {
            FileUpdateInfo fileUpdateInfo = fileUpdateInfoService.getFileById(id);
            if (fileUpdateInfo == null) {
                return ApiResult.error("未找到对应的文件信息，ID: " + id);
            }
            String fileName = fileUpdateInfo.getFileName();
            controlApplication(fileName, action);
            return ApiResult.success("服务控制命令已发送: " + action);
        } catch (Exception e) {
            return ApiResult.error("控制服务失败: " + e.getMessage());
        }
    }

    private void controlApplication(String fileName, String action) throws Exception {
        String osName = System.getProperty("os.name").toLowerCase();

        if (osName.contains("linux") || osName.contains("unix")) {
            // Linux 系统
            String command = "sh " + linuxContralScript + " " + action + " " + fileName;
            Runtime.getRuntime().exec(command);
        } else if (osName.contains("windows")) {
            // Windows 系统
            String command;
            switch (action.toLowerCase()) {
                case "start":
                    command = windowsStartScript + " " + fileName;
                    break;
                case "stop":
                    command = windowsStopScript + " " + fileName;
                    break;
                case "restart":
                    // Windows 系统没有单独的重启脚本，需要先停止再启动
                    Runtime.getRuntime().exec(windowsStopScript + " " + fileName);
                    Thread.sleep(5000); // 等待 5 秒，确保服务停止
                    Runtime.getRuntime().exec(windowsStartScript + " " + fileName);
                    return;
                default:
                    throw new IllegalArgumentException("不支持的操作类型: " + action);
            }
            Runtime.getRuntime().exec(command);
        } else {
            throw new Exception("不支持的操作系统: " + osName);
        }
    }
}